<?php

namespace Drupal\graphql_compose_configs\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for the Configuration Exposure add and edit forms.
 */
class ConfigurationExposureForm extends EntityForm {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a ConfigurationExposureForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $exposure = $this->entity;

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $exposure->label(),
      '#description' => $this->t('Label for the Configuration Exposure.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $exposure->id(),
      '#machine_name' => [
        'exists' => '\Drupal\graphql_compose_configs\Entity\ConfigurationExposure::load',
      ],
      '#disabled' => !$exposure->isNew(),
    ];

    // Get all available configurations.
    $configs = [];
    foreach ($this->configFactory->listAll() as $config_name) {
      $configs[$config_name] = $config_name;
    }

    $form['config_name'] = [
      '#type' => 'select',
      '#title' => $this->t('Configuration'),
      '#description' => $this->t('Select the configuration to expose.'),
      '#options' => $configs,
      '#default_value' => $exposure->getConfigName(),
      '#required' => TRUE,
      '#ajax' => [
        'callback' => '::updateAvailableFields',
        'wrapper' => 'available-fields-wrapper',
      ],
    ];

    $form['exposed_fields_wrapper'] = [
      '#type' => 'container',
      '#prefix' => '<div id="available-fields-wrapper">',
      '#suffix' => '</div>',
    ];

    $config_name = $form_state->getValue('config_name') ?: $exposure->getConfigName();
    if ($config_name) {
      $config = $this->configFactory->get($config_name);
      $data = $config->get();

      // Remove internal Drupal configuration keys.
      unset($data['_core']);

      $fields = $this->flattenArray($data);

      // Filter out any remaining internal keys.
      $fields = array_filter($fields, function($field) {
        return !str_starts_with($field, '_');
      });

      if (!empty($fields)) {
        // Create options array with original field path as key and display
        // value.
        $options = [];
        foreach ($fields as $field) {
          $options[$field] = $field;
        }

        // Decode the stored field values for display in the form.
        $default_values = [];
        foreach ($exposure->getExposedFields() as $field) {
          $default_values[] = $field;
        }

        $form['exposed_fields_wrapper']['exposed_fields'] = [
          '#type' => 'checkboxes',
          '#title' => $this->t('Fields to expose'),
          '#description' => $this->t('Select which fields from this configuration should be exposed in GraphQL.'),
          '#options' => $options,
          '#default_value' => $default_values,
        ];
      }
      else {
        $form['exposed_fields_wrapper']['message'] = [
          '#type' => 'markup',
          '#markup' => $this->t('No exposable fields found in this configuration.'),
        ];
      }
    }

    return $form;
  }

  /**
   * Ajax callback to update available fields based on selected configuration.
   */
  public function updateAvailableFields(array $form, FormStateInterface $form_state) {
    return $form['exposed_fields_wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    // Filter out unselected checkboxes.
    $exposed_fields = array_filter($form_state->getValue('exposed_fields'));

    // Store the selected fields.
    $this->entity->set('exposed_fields', array_values($exposed_fields));
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $exposure = $this->entity;
    $status = $exposure->save();

    if ($status) {
      $this->messenger()->addMessage($this->t('Saved the %label Configuration Exposure.', [
        '%label' => $exposure->label(),
      ]));
    }
    else {
      $this->messenger()->addMessage($this->t('The %label Configuration Exposure was not saved.', [
        '%label' => $exposure->label(),
      ]), 'error');
    }

    $form_state->setRedirectUrl($exposure->toUrl('collection'));
  }

  /**
   * Flattens a nested array into dot notation.
   *
   * @param array $array
   *   The array to flatten.
   * @param string $prefix
   *   The prefix to use for keys.
   *
   * @return array
   *   The flattened array.
   */
  protected function flattenArray(array $array, $prefix = '') {
    $result = [];
    foreach ($array as $key => $value) {
      if (is_array($value)) {
        $result = array_merge($result, $this->flattenArray($value, $prefix . $key . '.'));
      }
      else {
        $result[] = $prefix . $key;
      }
    }
    return $result;
  }

}
