<?php

namespace Drupal\graphql_compose_configs\Plugin\GraphQL\DataProducer;

use Drupal\Core\Config\Schema\TypedConfigInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @DataProducer(
 *   id = "config_data",
 *   name = @Translation("Configuration data"),
 *   description = @Translation("Returns configuration data for a given config name."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Configuration data")
 *   ),
 *   consumes = {
 *     "config_name" = @ContextDefinition("string",
 *       label = @Translation("Configuration name")
 *     ),
 *     "field_path" = @ContextDefinition("string",
 *       label = @Translation("Field path"),
 *       required = false
 *     ),
 *     "langcode" = @ContextDefinition("string",
 *       label = @Translation("Language code"),
 *       required = false
 *     )
 *   }
 * )
 */
class ConfigData extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The typed config manager.
   *
   * @var \Drupal\Core\Config\TypedConfigManagerInterface
   */
  protected $typedConfigManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->configFactory = $container->get('config.factory');
    $instance->typedConfigManager = $container->get('config.typed');
    $instance->languageManager = $container->get('language_manager');
    return $instance;
  }

  /**
   * Resolves the configuration data.
   *
   * @param string $config_name
   *   The configuration name.
   * @param string|null $field_path
   *   The field path to retrieve, if any.
   * @param string|null $langcode
   *   The language code to retrieve the configuration in.
   *
   * @return mixed
   *   The configuration data.
   */
  public function resolve(string $config_name, $field_path = NULL, $langcode = NULL) {
    $value = NULL;
    if ($langcode && $this->languageManager->isMultilingual()) {
      try {
        // Store the original language.
        $original_language = $this->languageManager->getConfigOverrideLanguage();

        // Get the language object for the given langcode.
        $language = $this->languageManager->getLanguage($langcode);

        // Set the config override language.
        $this->languageManager->setConfigOverrideLanguage($language);

        // Get the configuration value.
        $config = $this->configFactory->get($config_name);
        $value = $field_path ? $config->get($field_path) : $config->get();

        // Restore the original language.
        $this->languageManager->setConfigOverrideLanguage($original_language);
      }
      catch (\Exception $e) {
        // In case of errors, fall back to the default language.
        $config = $this->configFactory->get($config_name);
        $value = $field_path ? $config->get($field_path) : $config->get();
      }
    }
    else {
      // Get the configuration value in the default language.
      $config = $this->configFactory->get($config_name);
      $value = $field_path ? $config->get($field_path) : $config->get();
    }

    // Get the typed configuration to determine the schema type.
    if ($typed_config = $this->getTypedConfig($config_name, $field_path)) {
      // DataType objects will have a getValue method.
      if (method_exists($typed_config, 'getValue')) {
        return $typed_config->getValue();
      }
    }

    return $value;
  }

  /**
   * Gets a typed configuration object for the given path.
   *
   * @param string $config_name
   *   The configuration name.
   * @param string|null $field_path
   *   The field path, if any.
   *
   * @return \Drupal\Core\Config\Schema\TypedConfigInterface|null
   *   The typed configuration object, or NULL if not found.
   */
  protected function getTypedConfig(string $config_name, $field_path = NULL) {
    // Get the typed configuration.
    $typed_config = $this->typedConfigManager->get($config_name);

    if ($typed_config && $field_path) {
      try {
        // Dive into nested field paths.
        $parts = explode('.', $field_path);
        foreach ($parts as $part) {
          if ($typed_config instanceof TypedConfigInterface && $typed_config->get($part) instanceof TypedConfigInterface) {
            $typed_config = $typed_config->get($part);
          }
          else {
            return NULL;
          }
        }
      }
      catch (\Exception $e) {
        return NULL;
      }
    }

    return $typed_config;
  }

}
