<?php

namespace Drupal\graphql_compose_configs\Plugin\GraphQLCompose\SchemaType;

use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeSchemaTypeBase;
use Drupal\graphql_compose_configs\GraphQLConfigNamingTrait;
use GraphQL\Type\Definition\ObjectType;
use GraphQL\Type\Definition\Type;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * {@inheritdoc}
 *
 * @GraphQLComposeSchemaType(
 *   id = "DrupalConfig",
 * )
 */
class DrupalConfig extends GraphQLComposeSchemaTypeBase {

  use GraphQLConfigNamingTrait;

  /**
   * The typed config manager.
   *
   * @var \Drupal\Core\Config\TypedConfigManagerInterface
   */
  protected TypedConfigManagerInterface $typedConfigManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->typedConfigManager = $container->get('config.typed');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getTypes(): array {
    $types = [];

    // Load all configuration exposures.
    $storage = $this->entityTypeManager->getStorage('graphql_config_exposure');
    $exposures = $storage->loadMultiple();

    foreach ($exposures as $exposure) {
      /** @var \Drupal\graphql_compose_configs\Entity\ConfigurationExposure $exposure */
      $config_name = $exposure->getConfigName();
      $exposed_fields = $exposure->getExposedFields();
      $type_name = $this->createSchemaTypeNameFromConfigName($config_name);

      $fields = [];
      foreach ($exposed_fields as $field) {
        $graphql_field_name = $this->createFieldNameFromPath($field);
        $graphql_type = $this->determineGraphQLType($config_name, $field);

        $fields[$graphql_field_name] = [
          'type' => $graphql_type,
          'description' => sprintf('Configuration value for %s', $field),
        ];
      }

      $types[] = new ObjectType([
        'name' => $type_name,
        'description' => sprintf('Configuration object for %s', $config_name),
        'fields' => fn() => $fields,
      ]);
    }

    return $types;
  }

  /**
   * {@inheritdoc}
   */
  public function getExtensions(): array {
    $extensions = [];

    // Load all configuration exposures.
    $storage = $this->entityTypeManager->getStorage('graphql_config_exposure');
    $exposures = $storage->loadMultiple();

    // Define fields for Query type extension.
    $query_fields = [];
    foreach ($exposures as $exposure) {
      /** @var \Drupal\graphql_compose_configs\Entity\ConfigurationExposure $exposure */
      $config_name = $exposure->getConfigName();
      $field_name = $this->createFieldNameFromPath($config_name);
      $type_name = $this->createSchemaTypeNameFromConfigName($config_name);

      $query_fields[$field_name] = [
        'type' => static::type($type_name),
        'description' => sprintf('Access configuration for %s', $config_name),
        'args' => [
          'langcode' => [
            'type' => Type::string(),
            'description' => 'The language code to return the configuration in.',
          ],
        ],
      ];
    }

    if (!empty($query_fields)) {
      $extensions[] = new ObjectType([
        'name' => 'Query',
        'fields' => fn() => $query_fields,
      ]);
    }

    return $extensions;
  }

  /**
   * Determines the GraphQL type based on Drupal's config schema.
   *
   * @param string $config_name
   *   The configuration name.
   * @param string $field_path
   *   The field path.
   *
   * @return \GraphQL\Type\Definition\Type
   *   The GraphQL type for the field.
   */
  protected function determineGraphQLType(string $config_name, string $field_path): Type {
    try {
      // Get a typed config object for the whole config.
      $typed_config = $this->typedConfigManager->get($config_name);

      if ($typed_config instanceof \Drupal\Core\Config\Schema\TypedConfigInterface && $field_path) {
        try {
          // The get() method can access nested properties with dot notation.
          $typed_config = $typed_config->get($field_path);
        }
        catch (\InvalidArgumentException $e) {
          // If the property doesn't exist, default to string.
          return Type::string();
        }
      }

      // Get the schema definition.
      if ($typed_config instanceof \Drupal\Core\TypedData\TypedDataInterface) {
        $definition = $typed_config->getDataDefinition();
        if (isset($definition['type'])) {
          return $this->mapDrupalTypeToGraphQL($definition['type']);
        }
      }
    }
    catch (\Exception $e) {
      // If anything goes wrong, default to string.
    }

    // Default to string if no type could be determined.
    return Type::string();
  }

  /**
   * Maps a Drupal config schema type to a GraphQL type.
   *
   * @param string $field_type
   *   The Drupal field type from the config schema.
   *
   * @return \GraphQL\Type\Definition\Type
   *   The corresponding GraphQL type.
   */
  protected function mapDrupalTypeToGraphQL(string $field_type): Type {
    switch ($field_type) {
      case 'boolean':
        return Type::boolean();

      case 'integer':
        return Type::int();

      case 'float':
        return Type::float();

      case 'sequence':
        return Type::listOf(Type::string());

      default:
        return Type::string();
    }
  }

}
